import java.math.BigInteger;
import java.util.ArrayList;


public class Party {
	private int num ;
	private ArrayList<Player> member ;
	private String entryPlayerString ;
	private BigInteger totalOverKillDamage ;
	private int totalPush ;
	private int entryPlayerBuildCount ;
	// それぞれのユニットのビルドが行われた回数
	private int[] buildCount ;

	
	public Party( int num, String playerString ) throws Exception{
		this.num = num ;
		member = new ArrayList<Player>() ;
		entryPlayerString = playerString ;
		totalOverKillDamage = BigInteger.ZERO ;
		totalPush = 0 ;
		entryPlayerBuildCount = 1 ;
		buildCount = new int[Player.publicStrage.length];
		member.add( new Player(0,entryPlayerString));
		
	}
	
	/**
	 * メンバー全員のステータスを出力します。
	 * @param numFlag
	 * @param commentFlag
	 * @param basedFlag
	 */
	public void printPartyStatus( boolean numFlag, boolean commentFlag ){
		for(int i=0;i<member.size();i++)
			member.get(i).printStatus(numFlag, commentFlag );
	}
	
	/**
	 * メンバーのアクションを未使用状態にします。
	 */
	public void resetPartyActions(){
		for(int i=0;i<member.size();i++){
			Player p = member.get(i) ;
			for(int j=0;j<p.getActions().size();j++)
				p.getActions().get(j).setUsed(false);
		}
	}
	
	
	
	/**
	 * 死亡ユニットに対する退場処理を行います。
	 */
	public void checkLeaveParty(){
		for(int i=0;i<member.size();i++){
			Player p = member.get(i);
			if( p.getHp().compareTo(BigInteger.ZERO) <= 0 )
				p.checkLife() ;
			if( p.getHp().compareTo(BigInteger.ZERO) <= 0 ){
				totalPush += p.getPush() ;
				totalOverKillDamage = totalOverKillDamage.add(p.getHp())  ;
				member.remove(p);
				i -- ;
			}
		}
	}
	
	/**
	 * メンバー全体にAPをセットします。
	 */
	public void setPartyAp(){
		for(int i=0;i<member.size();i++){
			Player p = member.get(i) ;
			if( p.getSleep() > 0 ){
				p.setSleep( p.getSleep()-1 ) ;
				p.setAp( BigInteger.ZERO ) ;
				p.setSubAp( BigInteger.ZERO );
				Util.outputString(p.getComment()+"は眠っている！",SettingMatch.ResultFileName);
				p.printStatus(false, true);
			}
			else
				p.setAp( p.getSpd() ) ;
		}
	}
	
	
	public int getNum() {
		return num;
	}

	public void setNum(int num) {
		this.num = num;
	}

	public ArrayList<Player> getMember() {
		return member;
	}
	public void setMember(ArrayList<Player> member) {
		this.member = member;
	}
	public String getEntryPlayerString() {
		return entryPlayerString;
	}
	public void setEntryPlayerString(String entryPlayerString) {
		this.entryPlayerString = entryPlayerString;
	}
	public BigInteger getTotalOverKillDamage() {
		return totalOverKillDamage;
	}
	public void setTotalOverKillDamage(BigInteger totalOverKillDamage) {
		this.totalOverKillDamage = totalOverKillDamage;
	}
	public int getTotalPush() {
		return totalPush;
	}
	public void setTotalPush(int totalPush) {
		this.totalPush = totalPush;
	}

	public int getEntryPlayerBuildCount() {
		return entryPlayerBuildCount;
	}

	public void setEntryPlayerBuildCount(int entryPlayerBuildCount) {
		this.entryPlayerBuildCount = entryPlayerBuildCount;
	}

	public int[] getBuildCount() {
		return buildCount;
	}

	public void setBuildCount(int[] buildCount) {
		this.buildCount = buildCount;
	}
}
