import java.math.BigInteger;
import java.util.ArrayList;


public class Player {
/*
 * 攻撃、防御、剣の種類などを保存
 */
	private int num ;
	private int baseHp, baseAtk, baseDef, baseSpd ;
	private BigInteger hp, maxHp, atk, def, spd, ap ;
	private ArrayList<Action> actions ;
	private int  poison, stan, shield, reflection, life, soul ;
	private int sleep, deadly ;
	private int push, quick ;
	private String comment = null ;
	private String actionStr = null ;
	private String masterStr ;
	private boolean dualActionFlag ;
	private Action useAction ;
	private BigInteger subAp ;
	// 剣師登場ターン
	private int appearTurn ;

	public static final String[] publicStrage =
	{
		"5/0/0/0//バブル",
		"20/0/0/0//フェンス",
		"35/0/0/0//ゲート",
		"50/0/0/0//ウォール",
		"10/0/0/1/鏡/アイシア",
		"15/0/0/2/散鏡/ミスティア",
		"20/0/0/3/魂散鏡/ウンディーネ",
		"15/0/0/2/魂速護熱四/グレベルニア・エノレリテレクシア",
		"0/3/0/3/高臥高牙/不動の造形師スフルトヴキニ",
		"5/0/0/3/祈祈霧熱二牙/赤霞",
		"5/0/0/3/祈祈霧熱二牙/アルメリア",
		"10/0/0/3/臥魂怒熱三/クーベルチュール",
		"5/0/0/3/熱牙祈祈霧盗/遠流",
		"5/0/0/2/儀涙詠祈祈霧押/エンプティー",
		"5/0/0/3/祈祈霧熱熱二/クロイツさん",
		"5/0/0/3/祈詠柵押増鏡/ミラージュ",
		"5/0/0/3/祈詠柵増鏡押/晶獣鬼パープルフリーメル",
		
	};

	/**
	 * コンストラクタ
	 * @param num　SettingMatch内でのユニット定義番号
	 * @param unitData　ユニット表記文字列
	 * @throws Exception 
	 */
	public Player(int i, String unitData) throws Exception{
		String[] value = unitData.split("/");
		if( value.length < 3 || value.length > 6 )
			throw new Exception("剣師のフォーマットが正しくありません。 / の数が不正です。\n"+unitData);

		try{
			// パターンすらなし
			if( value.length == 4 ){
				PlayerBuilder(i,Integer.parseInt(value[0]),Integer.parseInt(value[1]),
						Integer.parseInt(value[2]),Integer.parseInt(value[3]),"","");
			}
			// 名前なし
			else if( value.length == 5 ){
				PlayerBuilder(i,Integer.parseInt(value[0]),Integer.parseInt(value[1]),
						Integer.parseInt(value[2]),Integer.parseInt(value[3]),value[4],"");
			}
			// 名前あり
			else if( value.length == 6 ){
				PlayerBuilder(i,Integer.parseInt(value[0]),Integer.parseInt(value[1]),
						Integer.parseInt(value[2]),Integer.parseInt(value[3]),value[4],value[5]);
			}
		} catch (NumberFormatException nfe){
			throw new Exception("剣師のフォーマットが正しくありません。\n"+unitData);
		}
	}
	
	/**
	 * このオブジェクトをm100のユニットにします。
	 * @param num　SettingMatch内でのユニット定義番号
	 * @param hp　HP
	 * @param atk　攻撃力
	 * @param def　防御力
	 * @param spd　素早さ
	 * @param pattern　行動パターン
	 * @throws Exception 
	 */
	private void PlayerBuilder(int num , int hp, int atk, int def, int spd, String actionStr, String comment) throws Exception{
		this.num = num ;
		this.baseHp = hp ;
		baseAtk = atk ;
		baseDef = def ;
		baseSpd = spd ;
		this.hp = this.maxHp = new BigInteger(hp+"") ;
		this.atk = new BigInteger(atk+"") ;
		this.def = new BigInteger(def+"") ;
		this.ap = this.spd = new BigInteger(spd+"") ;
		this.subAp = BigInteger.ZERO ;
		shield = reflection = life = 0 ;
		this.actionStr = actionStr ;
		this.comment = comment ;
		actions = createActions(actionStr);
		this.masterStr = getEntryString() ;
	}

	/**
	 * 文字列から、Action配列を生成します。
	 * @param actionStr
	 * @return
	 * @throws Exception 
	 */
	private ArrayList<Action> createActions(String actionStr) throws Exception{
		if( actionStr == null )
			return new ArrayList<Action>() ;
		// アクションパターン生成
		ArrayList<Action> entryActions = new ArrayList<Action>() ;
		for(int i=0;i<actionStr.length();i++){ 
			String str = actionStr.substring(i, i+1) ;
			Action a = new Action(str);
			entryActions.add( a ) ;
			if( a.getType() == -1 )
				System.out.println("!");
		}
		return entryActions;
	}
	
	/**
	 * 基本登録剣師の文字列を返します。
	 * @return
	 */
	public String getEntryString(){
		return baseHp+"/"+baseAtk+"/"+baseDef+"/"+baseSpd+"/"+actionStr+"/"+comment ;
	}


	/**
	 * プレイヤーの状態を表示します。
	 */
	public void printStatus( boolean numFlag, boolean commentFlag){
		if( numFlag ){
			if(num<10)Util.outputString(" " ,SettingMatch.ResultFileName);
			Util.outputString("  "+num+": " ,SettingMatch.ResultFileName);
		}

		Util.outputString(hp+"/"+atk+"/"+def+"/"+spd+"/" ,SettingMatch.ResultFileName);

		boolean usedFlag = false ;
		for(int i=0;i<actions.size();i++){
			Action a = getActions().get(i);
			if( !a.isUsed() && usedFlag ){
				usedFlag = false ;
				Util.outputString( "]" ,SettingMatch.ResultFileName );
			}
			if( a.isUsed() && !usedFlag ){
				usedFlag = true ;
				Util.outputString( "[" ,SettingMatch.ResultFileName );
			}
			
			Util.outputString( a.getStates() ,SettingMatch.ResultFileName );

			if( a.isUsed() && i == actions.size()-1 )
				Util.outputString( "]" ,SettingMatch.ResultFileName );
		}

		if( commentFlag ){
			if( comment != null)
				Util.outputString("/"+comment ,SettingMatch.ResultFileName);
			else
				Util.outputString("/"+num ,SettingMatch.ResultFileName);
		}
		if(stan>0||sleep>0||deadly>0||poison>0
		 ||reflection>0||shield>0||soul>0||life>0||push>0||quick>0)
			Util.outputString("/<" ,SettingMatch.ResultFileName);
		if(stan>0)Util.outputString("衝" ,SettingMatch.ResultFileName);
		if(stan>1)Util.outputString("*"+stan+"," ,SettingMatch.ResultFileName);
		if(sleep>0)Util.outputString("眠" ,SettingMatch.ResultFileName);
		if(sleep>1)Util.outputString("*"+sleep+"," ,SettingMatch.ResultFileName);
		if(deadly>0)Util.outputString("猛" ,SettingMatch.ResultFileName);
		if(deadly>1)Util.outputString("*"+deadly+"," ,SettingMatch.ResultFileName);
		if(poison>0)Util.outputString("毒" ,SettingMatch.ResultFileName);
		if(poison>1)Util.outputString("*"+poison+"," ,SettingMatch.ResultFileName);
		if(reflection>0)Util.outputString("反" ,SettingMatch.ResultFileName);
		if(reflection>1)Util.outputString("*"+reflection+"," ,SettingMatch.ResultFileName);
		if(shield>0)Util.outputString("盾" ,SettingMatch.ResultFileName);
		if(shield>1)Util.outputString("*"+shield+"," ,SettingMatch.ResultFileName);
		if(soul>0)Util.outputString("魂" ,SettingMatch.ResultFileName);
		if(soul>1)Util.outputString("*"+soul+"," ,SettingMatch.ResultFileName);
		if(life>0)Util.outputString("命" ,SettingMatch.ResultFileName);
		if(life>1)Util.outputString("*"+life+"," ,SettingMatch.ResultFileName);
		if(push>0)Util.outputString("押" ,SettingMatch.ResultFileName);
		if(push>1)Util.outputString("*"+push+"," ,SettingMatch.ResultFileName);
		if(quick>0)Util.outputString("急" ,SettingMatch.ResultFileName);
		if(quick>1)Util.outputString("*"+quick+"," ,SettingMatch.ResultFileName);
		if(stan>0||sleep>0||deadly>0||poison>0
		 ||reflection>0||shield>0||soul>0||life>0||push>0||quick>0)
			Util.outputString(">" ,SettingMatch.ResultFileName);
		
		Util.outputString("\n" ,SettingMatch.ResultFileName);
	
	}
	
	/**
	 * 次に使うアクションを探します。
	 * @param turn
	 */
	public int searchNextAction( int turn ){
		// 通常のサーチ
		for(int select=0;select<actions.size();select++){
			Action a = actions.get(select);
			if( a.getType() != -1 && a.isUsed() == false
			 && a.getBoot() <= turn ){
				if( a.isPinch() && hp.compareTo( maxHp.divide(BigInteger.valueOf(2)) ) > 0 )
					continue ;
				else if( a.isSymptom() && stan==0 && poison==0 && deadly==0 && sleep==0 )
					continue ;
				else
					return select ;
			}
		}
		return -1 ;
	}
	/**
	 * スタンであるかをチェックし、スタンの場合はスタンを減らします。
	 */
	public boolean checkStan(){
		if( stan > 0 ){
			Util.outputString("    --"+getComment()+" スタン発動。行動見送り。\n" ,SettingMatch.ResultFileName);
			stan -- ;
			return true ;
		}
		return false ;
	}
	
	
	public void damage( BigInteger d, int turn ){
		BigInteger damage = d ;
		
		if( damage.compareTo(BigInteger.ZERO) <= 0 ){
			Util.outputString( "      --"+comment+"にはダメージを与えられない！\n" ,SettingMatch.ResultFileName);
			return ;
		}
		Util.outputString( "      --"+damage+"点のダメージ！  " ,SettingMatch.ResultFileName);
		hp = hp.subtract(damage) ;
		printStatus(false, true);
		if( hp.compareTo(BigInteger.ZERO) <= 0 && turn > 0 ){
			if( soul > 0 ){
				soul -- ;
				hp = maxHp ;
				Util.outputString("    --"+comment+" ソウル発動。　" ,SettingMatch.ResultFileName);
			}
			else if( life > 0 ){
				life -- ;
				hp = BigInteger.ONE ;
				Util.outputString("    --"+comment+" ライフ発動。　" ,SettingMatch.ResultFileName);
			}
		}
		
		
	}
	
	/**
	 * ライフの判定をします。生き返ったらHP1のライフ-1します。
	 * @param a
	 */
	public void checkLife(){
		if( getHp().compareTo(BigInteger.ZERO) <= 0 && getSoul() > 0 ){
			setHp(getMaxHp()) ;
			Util.outputString("    --"+getComment()+" ソウル発動。　" ,SettingMatch.ResultFileName);
			addSoul(-1) ;
			if( getHp().compareTo(BigInteger.ZERO) == 0 )
				setSoul(0);
			printStatus(false, false);
		}
		if( getHp().compareTo(BigInteger.ZERO) <= 0 && getLife() > 0 ){
			setHp(BigInteger.ONE) ;
			Util.outputString("    --"+getComment()+" ライフ発動。　" ,SettingMatch.ResultFileName);
			addLife(-1) ;
			printStatus(false, false);
		}
	}

	public void healLife( int turn ) {
		Util.outputString("    --"+getComment()+"のHPは全回復した。" ,SettingMatch.ResultFileName);
		this.hp = this.maxHp;
		printStatus(false, false);
		if( turn != 0 )
			checkLife();
	}
	
	/**
	 * 次に使用するアクションを選択し、保存します。
	 * @param maxSpd
	 * @param maxQuick
	 * @param turn
	 * @param myParty
	 * @param enemyParty
	 * @return 次に使用するアクション/スタンが存在しない場合、falseを返し、それ以外の場合でtrueを返します。
	 */
	public boolean setUseAction( BigInteger maxSpd, int maxQuick, int turn, Party myParty, Party enemyParty ){
		dualActionFlag = false ;
		useAction = null ;
		if( ap.add(subAp).compareTo(BigInteger.ZERO) > 0
		 && spd.compareTo(maxSpd) == 0 && quick == maxQuick ){
			// AP表示
			Util.outputString("  --"+comment+", 残りAP:"+ap ,SettingMatch.ResultFileName);
			if( subAp.compareTo(BigInteger.ZERO) > 0 )
				Util.outputString(" + 予備AP:"+subAp ,SettingMatch.ResultFileName);
			
			// スタンの場合
			if( stan > 0 ){
				Util.outputString("(スタン！)\n" ,SettingMatch.ResultFileName);
				stan -- ;
				if( ap.compareTo(BigInteger.ZERO) > 0 )
					ap = ap.subtract(BigInteger.ONE) ;
				else 
					subAp = subAp.subtract(BigInteger.ONE) ;
				return true ;
			}
			// pの使うアクションをuseActionに保存
			else{
				Util.outputString("\n" ,SettingMatch.ResultFileName);
				int select = -1 ;
				// pの次に使うアクションを探す
				if( (select = searchNextAction(turn)) != -1 ){
					Action action = actions.get( select ) ;
					// コスト支払い（使用済み化、MP消費
					action.setUsed(true);
					dualActionFlag = true ;
					useAction = action ;
					action.setTargetAction( null );
					if( actions.size() > select+1 )
						action.setTargetAction( actions.get(select+1) );
					if( action.isLimited() )
						actions.remove(action);

					if( action.getGenre() == Action.ATK )
						action.setTargetPlayers(searchTargetList(enemyParty.getMember(),true,action.isWhole())) ;
					else if( action.getGenre() == Action.SPT ){
						if( action.isWhole() )
							action.setTargetPlayers(searchTargetList(myParty.getMember(),false,action.isWhole())) ;
						else{
							ArrayList<Player> players = new ArrayList<Player>(1);
							players.add(this);
							action.setTargetPlayers(players);
						}
					}
					if( ap.compareTo(BigInteger.ZERO) > 0 )
						ap = ap.subtract(BigInteger.ONE) ;
					else 
						subAp = subAp.subtract(BigInteger.ONE) ;

					return true ;
				}
			}
			if( ap.compareTo(BigInteger.ZERO) > 0 )
				ap = ap.subtract(BigInteger.ONE) ;
		}
		return false ;
	}

	/**
	 * partyから、対象となるプレイヤーのListを返します。
	 * @param party
	 * @param atkFlag　攻撃アクションか（挑発判定）
	 * @param wholeFlag　アクションに全体化が付与されているか
	 * @return
	 */
	private ArrayList<Player> searchTargetList( ArrayList<Player> party, boolean atkFlag, boolean wholeFlag) {
		ArrayList<Player> targets = new ArrayList<Player>() ;
		// 全体化の場合
		if( wholeFlag ){
			for(int i=0;i<party.size();i++)
				targets.add(party.get(i)) ;
		}
		// 全体化なし
		else{
			targets.add(party.get(0));
		}
		
		return targets ;
	}


	public int getNum() {
		return this.num ;
	}
	public String getComment() {
		if(comment!=null)
			return comment;
		return "player:"+num ;
	}
	public void setComment(String comment) {
		this.comment = comment ;
	}
	public BigInteger getMaxHp() {
		return maxHp;
	}
	public void setMaxHp( BigInteger maxHp ) {
		this.maxHp = maxHp ;
	}
	public int getBaseHp() {
		return baseHp;
	}

	public void setBaseHp(int baseHp) {
		this.baseHp = baseHp;
	}

	public int getBaseAtk() {
		return baseAtk;
	}

	public int getBaseDef() {
		return baseDef;
	}

	public int getBaseSpd() {
		return baseSpd;
	}

	public BigInteger getHp() {
		return hp;
	}
	public void setHp(BigInteger hp) {
		this.hp = hp;
	}
	public BigInteger getAtk() {
		return atk;
	}
	public void setAtk(BigInteger atk) {
		this.atk = atk;
	}
	public void addAtk(BigInteger atk) {
		this.atk = this.atk.add(atk);
		if( this.atk.compareTo(BigInteger.ZERO) < 0 )
			this.atk = BigInteger.ZERO ;
	}
	public BigInteger getDef() {
		return def;
	}
	public void setDef(BigInteger def) {
		this.def = def;
	}
	public void addDef(BigInteger def) {
		this.def = this.def.add(def);
		if( this.def.compareTo(BigInteger.ZERO) < 0 )
			this.def = BigInteger.ZERO ;
	}
	public BigInteger getSpd() {
		return spd;
	}
	public void setSpd(BigInteger spd) {
		this.spd = spd;
	}
	public void addSpd(BigInteger spd) {
		this.spd = this.spd.add(spd);
	}
	public int getPoison() {
		return poison;
	}
	public void setPoison(int set) {
		this.poison = set ;
	}
	public void addPoison(int add) {
		this.poison += add ;
	}
	public int getStan() {
		return stan;
	}
	public void setStan(int set) {
		this.stan = set ;
	}
	public void addStan(int add) {
		this.stan += add ;
	}
	public int getShield() {
		return shield;
	}
	public void addShield(int add) {
		this.shield += add ;
	}
	public int getReflection() {
		return reflection;
	}
	public void setReflection(int reflection) {
		this.reflection = reflection ;
	}
	public void addReflection(int add) {
		this.reflection += add ;
	}
	public int getLife() {
		return life;
	}
	public void addLife(int add) {
		this.life += add ;
	}
	public int getSoul() {
		return soul;
	}

	public void setSoul(int soul) {
		this.soul = soul;
	}
	public void addSoul(int add) {
		this.soul += add ;
	}

	public ArrayList<Action> getActions() {
		return actions;
	}
	public void setActions(ArrayList<Action> actions) {
		this.actions = actions;
	}
	public String getActionStr() {
		return actionStr;
	}
	public void setActionStr(String actionStr) {
		this.actionStr = actionStr;
	}

	public String getMasterStr() {
		return masterStr;
	}
	public void setMasterStr(String masterStr) {
		this.masterStr = masterStr;
	}

	public BigInteger getAp() {
		return ap;
	}

	public void setAp(BigInteger ap) {
		this.ap = ap;
	}

	public int getSleep() {
		return sleep;
	}
	public void setSleep(int sleep) {
		this.sleep = sleep;
	}
	public void addSleep(int sleep) {
		this.sleep += sleep;
	}

	public int getDeadly() {
		return deadly;
	}
	public void setDeadly(int deadly) {
		this.deadly = deadly;
	}
	public void addDeadly(int deadly) {
		this.deadly += deadly;
	}
	
	public int getPush() {
		return push;
	}
	public void setPush(int push) {
		this.push = push;
	}
	public void addPush(int push) {
		this.push += push;
	}

	public int getQuick() {
		return quick;
	}
	public void setQuick(int quick) {
		this.quick = quick;
	}
	public void addQuick(int quick) {
		this.quick += quick;
	}

	public boolean isDualActionFlag() {
		return dualActionFlag;
	}

	public void setDualActionFlag(boolean dualActionFlag) {
		this.dualActionFlag = dualActionFlag;
	}

	public Action getUseAction() {
		return useAction;
	}

	public void setUseAction(Action useAction) {
		this.useAction = useAction;
	}

	public BigInteger getSubAp() {
		return subAp;
	}

	public void setSubAp(BigInteger subAp) {
		this.subAp = subAp;
	}

	public int getAppearTurn() {
		return appearTurn;
	}

	public void setAppearTurn(int appearTurn) {
		this.appearTurn = appearTurn;
	}

}