import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.math.BigInteger;
import java.util.ArrayList;

/**
 * 戦闘処理を行うためのベースクラスです。
 * @author theki
 */
public class SettingMatch {
	// 同一タグのグループ
	protected ArrayList<Group> groups = new ArrayList<Group>() ;
	// 現在のターン数
	private int turn = 0 ;
	// 詳細な処理を出力するかどうか
	protected boolean fileOutFlag = false ;
	// 戦闘結果の出力用ファイル名
	public static final String ResultFileName = "result.txt";
	public static String ResultTableFileName ;

	/**
	 * コンストラクタ
	 * fileNameを名前とするファイルより剣師一覧を取り込み、groupsを生成します。
	 * @param fileName
	 * @throws Exception
	 */
	public SettingMatch( String fileName, String className ) throws Exception {
		ResultTableFileName = className ;		
		BufferedReader fin  = new BufferedReader(new FileReader(fileName));
		String data;	// ファイル入力文字列
		String groupName = "" ;	// 現在のタグ名
		int heartMax = -1 ;		// 最大途心
		int heartMin = -1 ;		// 最小途心
		
		File deleteFile = new File(ResultFileName);				
		deleteFile.delete();
		deleteFile = new File(ResultTableFileName+".txt");
		deleteFile.delete();
		deleteFile = new File(ResultTableFileName+".csv");
		deleteFile.delete();
		
		while((data = fin.readLine()) != null){	
			data = data.replaceAll(" ", "");
			// コメントアウトモード
			if( data.startsWith("#") )
				continue ;
			else if( data.startsWith("heart_max=") ){
				if( heartMax != -1 )
					throw new Exception("\"heart_max\"の指定が複数存在します。");
				heartMax = Util.getNextNum(data.replaceFirst("heart_max", ""), 0);
			}
			else if( data.startsWith("heart_min=") ){
				if( heartMin != -1 )
					throw new Exception("\"heart_min\"の指定が複数存在します。");
				heartMin = Util.getNextNum(data.replaceFirst("heart_min", ""), 0);
			}
			// タグ付けモード
			else if( data.startsWith("[") && data.endsWith("]") )
				groupName = data.replaceFirst("\\[", "").replaceAll("\\]", "");
			
			// タグ剣師モード
			else if( data.lastIndexOf("/") < data.lastIndexOf("[") && data.endsWith("]") )
				AddPlayerInGroup(data.substring(0, data.lastIndexOf("[")), data.substring(data.lastIndexOf("[")+1,data.length()-1));
			
			// タグなし剣師モード
			else if( data.length() != 0 )
				AddPlayerInGroup(data, groupName);			
		}
		
		// 途心チェック
		for(int i=0;i<groups.size();i++){
			Group g = groups.get(i);
			for(int j=0;j<g.getPlayers().size();j++){
				Player p = g.getPlayers().get(j);
				BigInteger heart = p.getMaxHp().divide(BigInteger.valueOf(5)).add(p.getAtk())
									.add(p.getDef()).add(p.getSpd()).add(BigInteger.valueOf(p.getActions().size()));
				if( heartMax != -1 && BigInteger.valueOf(heartMax).compareTo(heart) < 0 )
					throw new Exception(p.getEntryString()+" の剣師は途心の上限をオーバーしています。");
				if( heartMin != -1 && BigInteger.valueOf(heartMin).compareTo(heart) > 0 )
					throw new Exception(p.getEntryString()+" の剣師は途心の下限をオーバーしています。");
			}
		}
	}
	
	/**
	 * groupNameを名前とするgroup内に、文字列dataに記述される剣師を追加します。
	 * groupNameに対応するgroupがない場合、新しく作成します。
	 * @param data 剣師文字列
	 * @param groupName グループ名
	 * @throws Exception 
	 */
	private void AddPlayerInGroup( String data, String groupName ) throws Exception{
		// groupNameに対応するgroupがあれば、groupにユニットを追加
		for(int i=0;i<groups.size();i++){
			if( groupName.equalsIgnoreCase(groups.get(i).getName()) ){
				groups.get(i).getPlayers().add( new Player(groups.get(i).getPlayers().size(), data ));
				return ;
			}
		}
		// 対応するgroupがなければ、新しくgroupを生成してユニットを追加
		groups.add( new Group(groupName) ) ;
		groups.get(groups.size()-1).getPlayers().add( new Player( 0, data ) ) ;
	}

	/**
	 * バトルの組み合わせや勝敗表の表示などを行います。
	 * @throws Exception 
	 */
	public void exec() throws Exception{
	}
	
	/**
	 * Player a と Player b のバトルの結果を返します。
	 * 引き分けなら-1、aの勝利なら1,bの勝利なら2を返します。
	 * @param a
	 * @param b
	 * @throws Exception 
	 */
	protected int battle( Player a, Player b ) throws Exception{
		turn = 0 ;
		// 両者のユニットリスト
		Party aParty = new Party( a.getNum(), a.getEntryString() ) ;
		Party bParty = new Party( b.getNum(), b.getEntryString() ) ;
		
		int judgeResult = 0 ;	// 勝敗判定

		// ターン毎の戦闘処理
		while( turn <= 20 ){
			
			// 剣を未使用状態にする
			aParty.resetPartyActions();
			bParty.resetPartyActions();

			// ターン開始処理
			Util.outputString("--turn:"+turn+"\n" ,ResultFileName );
			Util.outputString("--"+a.getComment()+"'s party\n" ,ResultFileName );
			aParty.printPartyStatus(false,true);
			Util.outputString("--"+b.getComment()+"'s party\n" ,ResultFileName );
			bParty.printPartyStatus(false,true);
			
			// 1ターン目開始の死亡判定
			if( turn == 1 && ( judgeResult = checkJudge( aParty, bParty, turn ) ) != 0 ){
				Util.outputString("\n  --決着。\n" ,ResultFileName );
				return judgeResult ;
			}

			// 毒判定
			checkPoison( aParty, bParty ) ;

			// 毒死亡判定
			if( ( judgeResult = checkJudge( aParty, bParty, turn ) ) != 0 ){
				Util.outputString("\n  --決着。\n" ,ResultFileName );
				return judgeResult ;
			}

			// APセット
			aParty.setPartyAp();
			bParty.setPartyAp();
			Util.outputString("\n" ,ResultFileName );
			
			// 行動権を獲得するユニットと、使用する剣を決定
			boolean turnEndFlag = entitleAct( aParty, bParty ) ;
			while( !turnEndFlag ){

				// 同時行動
				for(int i=0;i<=Action.ENV;i++)
					processingDualAction( i, aParty, bParty );

				// 勝敗判定
				if( ( judgeResult = checkJudge( aParty, bParty, turn ) ) != 0 ){
					Util.outputString("\n  --決着。\n" ,ResultFileName );
					return judgeResult ;
				}
				
				Util.outputString("\n" ,ResultFileName );
				// 次に同時行動を行うユニットと、使用する剣を決定
				turnEndFlag = entitleAct( aParty, bParty );

			}
			turn ++ ;
		}
		Util.outputString("  --終了ターンです。\n" ,ResultFileName );

		return checkJudge( aParty, bParty, turn );
	}
	
	/**
	 * 2つのパーティから、同時行動を行うユニットを1体ずつ取得し、返します。
	 * @param genre 剣の実行種別
	 * @param a
	 * @param b
	 * @return [パーティa側の同時行動剣師、パーティb側の同時行動剣師]
	 *         同時行動剣師がいない場合は剣師の代わりにnullが格納される
	 */
	private Player[] getDualActionPlayers( int genre, Party a, Party b ){
		Player[] result = {null,null};
		for(int i=0;i<a.getMember().size();i++){
			Player p = a.getMember().get(i);
			if( p.isDualActionFlag() && p.getUseAction().getGenre() == genre ){
				p.setDualActionFlag(false);
				result[0] = p ;
				break ;
			}
		}
		for(int i=0;i<b.getMember().size();i++){
			Player p = b.getMember().get(i);
			if( p.isDualActionFlag() && p.getUseAction().getGenre() == genre ){
				p.setDualActionFlag(false);
				result[1] = p ;
				break ;
			}
		}
		return result ;

	}
	
	/**
	 * 同時行動処理を行います。
	 * @param genre 剣の実行種別
	 * @param aParty
	 * @param bParty
	 * @throws Exception 
	 */
	private void processingDualAction( int genre, Party aParty, Party bParty ) throws Exception{
		final int SPT = 0, CTR = 1, ATK = 2,ENV = 3 ;
		Player[] dualActionPlayers = getDualActionPlayers( genre, aParty, bParty ) ;
		Player ap = dualActionPlayers[0] ;
		Player bp = dualActionPlayers[1] ;
		
		// 同時行動発生
		while( ap != null || bp != null ){
			Action aa = null ;	// 1の使用アクション
			Action ba = null ;	// 2の使用アクション

			// 対象となるユニットを抽出
			ArrayList<Player> aTargetList = new ArrayList<Player>() ;

			ArrayList<Player> bTargetList = new ArrayList<Player>() ;
			if( ap != null ){
				aa = ap.getUseAction() ;
				if( genre == SPT || genre == ATK )
					aTargetList = aa.getTargetPlayers() ;
			}
			if( bp != null ){
				ba = bp.getUseAction() ;
				if( genre == SPT || genre == ATK )
					bTargetList = ba.getTargetPlayers() ;
			}
			
			// 超同時行動
			if( aa != null && ba != null ){
				Util.outputString("    --超同時行動。\n" ,ResultFileName );
				Util.outputString("    --"+ap.getComment()+"の　"+Action.NAME[aa.getType()]+aa.getOptions()+"！ \n" ,ResultFileName );
				Util.outputString("    --"+bp.getComment()+"の　"+Action.NAME[ba.getType()]+ba.getOptions()+"！ \n" ,ResultFileName );

				// 初めからターゲットがいない（攻撃/補助以外）なら
				if( aTargetList.size() == 0 && bTargetList.size() == 0  ){
					// 制御
					if( genre == CTR ){
						aa.execControl(ap);
						ba.execControl(bp);
					}
					// ビルド＋環境
					else if( genre == ENV ){
						turn = aa.execBuildAndEnv(ap, aParty, bParty, turn);
						turn = ba.execBuildAndEnv(bp, bParty, aParty, turn);
					}
				}
				
				// 攻撃/補助なら、どちらもターゲットがいなくなるまで
				while( aTargetList.size() > 0 || bTargetList.size() > 0 ){
					// ターゲットがいるなら行動
					Player at = null ;
					Player bt = null ;
					if( aTargetList.size() > 0 ){
						at = aTargetList.get(0);
						aTargetList.remove(at);
					}
					if( bTargetList.size() > 0 ){
						bt = bTargetList.get(0);
						bTargetList.remove(bt);
					}
					
					// 連撃の回数だけ繰り返す
					boolean aFlag = false ;
					boolean bFlag = false ;
					if( at != null )
						aFlag = true ;
					if( bt != null )
						bFlag = true ;
					for(int hit=0; aFlag || bFlag ;hit++){

						// 攻撃の場合は反射処理を考慮する
						if( genre == ATK ){
							if( aFlag && bFlag ){
								// 超超同時行動の判定
								Player[] p = checkReflection(ap, at, bp, bt, 0, aa.isIgnoreMirror(), ba.isIgnoreMirror());
								
								// 超超同時行動の発生
								if( p[4] == null )
									superSuperDualAction(aa,ba,p[0],p[1],p[2],p[3],turn);
								// aが先に行動。
								else if( p[4].equals(ap) ){
									processingAttack(p[0], p[1], aa);
									processingAttack(p[2], p[3], ba);
								}
								// aが後に行動。
								else{
									processingAttack(p[2], p[3], ba);
									processingAttack(p[0], p[1], aa);
								}
							}
							// 攻撃・片方のみ
							else if( aFlag )
								processingAttack(ap, checkReflection(ap, at, aa.isIgnoreMirror() ), aa);
							else if( bFlag )
								processingAttack(bp, checkReflection(bp, bt, ba.isIgnoreMirror() ), ba);

						// 補助ならば反射考慮なし
						} else {
							if( aFlag )
								aa.execSupport(ap,at, turn);
							if( bFlag )
								ba.execSupport(bp,bt, turn);
						}
						
						// 次で連撃終了なら、フラグ閉じる
						if( hit+1 == aa.getHit() ) aFlag = false ;
						if( hit+1 == ba.getHit() ) bFlag = false ;
					}
					
				}

			// 単独行動
			} else if( aa != null || ba != null ) {
				// 必要なデータを置き換え
				ArrayList<Player> targetList ;
				Party party ;
				Party eParty ;
				Player p,t ;
				Action a = null ;
				if( ap != null ){
					p = ap ;
					targetList = aTargetList ;
					a = aa ;
					party = aParty ;
					eParty = bParty ;
				}
				else{
					p = bp ;
					targetList = bTargetList ;
					a = ba ;
					party = bParty ;
					eParty = aParty ;
				}
				Util.outputString("    --"+p.getComment()+"の　"+Action.NAME[a.getType()]+a.getOptions()+"！ \n" ,ResultFileName );

				// 初めからターゲットがいない（攻撃/補助以外）なら
				if( targetList.size() == 0 ){
					// 制御
					if( genre == CTR )
						a.execControl(p);
						
					// ビルド＋環境
					else if( genre == ENV )
						turn = a.execBuildAndEnv(p, party, eParty, turn);
				}
				
				// どちらもターゲットがいなくなるまで
				while( targetList.size() > 0 ){
					t = targetList.get(0);
					targetList.remove(t);
					
					// 連撃の回数だけ繰り返す
					for(int hit=0; hit < a.getHit() ;hit++){
						// 攻撃の場合は反射処理
						if( genre == ATK ){
							processingAttack(p, checkReflection(p, t, a.isIgnoreMirror() ), a);
						} else {
							a.execSupport(p,t, turn) ;
						}
						
					}
					
				}
				
				// ターンが20を過ぎてたらの処理
				if( turn > 20 )
					return ;
			}
			if( aa != null ){
				ap.getUseAction().secondEffect(ap,aa,turn);
			}
			if( ba != null ){
				bp.getUseAction().secondEffect(bp,ba,turn);
			}
			
			// 行動するユニットを更新
			dualActionPlayers = getDualActionPlayers( genre, aParty, bParty ) ;
			ap = dualActionPlayers[0] ;
			bp = dualActionPlayers[1] ;


		}
	}

	/**
	 * 反射効果を含んだ、攻撃時の処理を行います。
	 * @param a 行動剣師
	 * @param t 攻撃対象剣師
	 * @param action 攻撃剣
	 */
	private void processingAttack( Player a, Player t, Action action ){

		BigInteger damage ; 

		damage = action.execAttack(a,t, turn) ;
		t.damage( damage, turn );
		action.additionEffect(a,t,action,damage,turn);
		Action rob = action.optionEffect(a,t,damage,turn);
		// 強盗処理
		if( rob != null ){
			t.getActions().remove(rob);
			a.getActions().add(rob);
			Util.outputString("      --"+t.getComment()+"の"+Action.NAME[rob.getType()]+"を強盗。\n" ,SettingMatch.ResultFileName);
		}
	}
	
	/**
	 * 超超同時行動の処理を行います。apがatにaaを、bpがbtにbbを使います。
	 * 見切りの処理もします。
	 * @param aa パーティa側の実行する剣
	 * @param ba パーティb側の実行する剣
	 * @param ap パーティa側の行動剣師
	 * @param at パーティa側の行動対象剣師
	 * @param bp パーティa側の行動剣師
	 * @param bt パーティb側の行動対象剣師
	 * @param turn 
	 */
	protected void superSuperDualAction(Action aa, Action ba, Player ap, Player at,
			Player bp, Player bt, int turn) {
		Action aRob = null ;
		Action bRob = null ;
		
		BigInteger aDamage = aa.execAttack(ap,at,turn) ;
		BigInteger bDamage = ba.execAttack(bp,bt,turn) ;
		at.damage( aDamage, turn );
		bt.damage( bDamage, turn );
		
		aa.additionEffect(ap,at,ba,aDamage,turn);
		ba.additionEffect(bp,bt,aa,bDamage,turn);
		aRob = aa.optionEffect(ap,at,aDamage,turn);
		bRob = ba.optionEffect(bp,bt,bDamage,turn);
		
		// 強盗処理
		if( aRob != null ){
			at.getActions().remove(aRob);
			ap.getActions().add(aRob);
			Util.outputString("      --"+at.getComment()+"の"+Action.NAME[aRob.getType()]+"を強盗。\n" ,SettingMatch.ResultFileName);
		}
		if( bRob != null ){
			bt.getActions().remove(bRob);
			bp.getActions().add(bRob);
			Util.outputString("      --"+bt.getComment()+"の"+Action.NAME[bRob.getType()]+"を強盗。\n" ,SettingMatch.ResultFileName);
		}
	}

	/**
	 * Player aとat、bとbtの間の同時行動における反射処理を解決します。
	 * @param a パーティa側の行動剣師
	 * @param at パーティa側の行動対象剣師
	 * @param b パーティa側の行動剣師
	 * @param bt パーティa側の行動対象剣師
	 * @return [a,aの反射後の対象,b,bの反射後の対象,先に行動するプレイヤー(a or b or null)]
	 *         先に行動するプレイヤーがnullの場合は超超同時行動
	 */
	protected Player[] checkReflection( Player a, Player at, Player b, Player bt, int tempo, boolean aMirrorFlag, boolean bMirrorFlag ){
		Player[] result = {a, at, b, bt, null} ;
		boolean aReflag = false ;
		boolean bReflag = false ;
		// atが反射
		if( at.getReflection() > 0 && !aMirrorFlag ){
			at.addReflection(-1) ;
			Util.outputString("    --"+at.getComment()+"　反射発動。　反射："+at.getReflection()+"\n" ,ResultFileName );
			aReflag = true ;
		}
		// btが反射
		if( bt.getReflection() > 0 && !bMirrorFlag ){
			bt.addReflection(-1) ;
			Util.outputString("    --"+bt.getComment()+"　反射発動。　反射："+bt.getReflection()+"\n" ,ResultFileName );
			bReflag = true ;
		}
		
		// どちらも反射なら続行
		if( aReflag && bReflag )
			return checkReflection( at, a, bt, b, tempo+1, aMirrorFlag, bMirrorFlag );
		// 片方だけ反射
		else if( aReflag ^ bReflag ){
			// 奇数回の反射なら、対象は実行者
			if( (tempo%2) == 1 ){
				result[2] = bt ;
				result[0] = at ;
			}
			if( aReflag ){
				result[1] = checkReflection(at,a,aMirrorFlag);
				result[4] = b ;
			}
			if( bReflag ){
				result[3] = checkReflection(bt,b,bMirrorFlag);
				result[4] = a ;
			}
			return result;
		}
		// どちらも反射がないならば超超同時行動
		else {
			// 同時反射で奇数回だった場合、行動者＝対象
			if( (tempo%2) == 1 ){
				result[0] = result[1] ;
				result[2] = result[3] ;
			}
			return result;
		}
	}
	
	/**
	 * Player aとatの間の反射を解決します。
	 * @param a 行動剣師
	 * @param at 行動対象剣師
	 * @return 最終的な対象となるPlayer
	 */
	protected Player checkReflection( Player a, Player at, boolean mirrorFlag ){
		Player result = at ;
		if( at.getReflection()>0 && !mirrorFlag ){
			at.addReflection(-1) ;
			Util.outputString("    --"+at.getComment()+"　反射発動。　反射："+at.getReflection()+"\n" ,ResultFileName );
			return checkReflection( at, a, mirrorFlag  ) ;
		}
		else
			return result ;
	}

	/**
	 * 死亡判定、及びに勝敗判定を行います。
	 * @param a
	 * @param b
	 * @return 勝敗がついていないなら0、引き分けなら-1、aの勝利なら1、bの勝利なら2
	 */
	protected int checkJudge( Party a, Party b, int turn ){
		// 0ターン目に勝敗判定はなし
		if( turn < 1 )
			return 0 ;
		
		a.checkLeaveParty() ;
		b.checkLeaveParty() ;
		
		boolean aFlag = true ;
		boolean bFlag = true ;
		ArrayList<Player> aParty = a.getMember() ;
		ArrayList<Player> bParty = b.getMember() ;
		
		// 死亡フラグ処理（trueなら全滅）
		if( aParty.size() != 0 )
			aFlag = false ;
		if( bParty.size() != 0 )
			bFlag = false ;

		// お互いが生存なら続行orHP判定
		if( !aFlag && !bFlag ){
			if( turn <= 20 )
				return 0 ;
			// HP判定処理
			int aPush , bPush ;
			BigInteger aHp , bHp ;
			aPush = bPush = 0 ;
			aHp = bHp = BigInteger.ZERO ;
			for(int i=0;i<aParty.size();i++)
				aPush += aParty.get(i).getPush() ;
			for(int i=0;i<bParty.size();i++)
				bPush += bParty.get(i).getPush() ;
			for(int i=0;i<aParty.size();i++)
				aHp = aHp.add(aParty.get(i).getHp()) ;
			for(int i=0;i<bParty.size();i++)
				bHp = bHp.add(bParty.get(i).getHp()) ;
			Util.outputString("  --判定：　<プッシュ>："+aPush+"　- "+bPush+", HP:"+aHp+" - "+bHp+"\n" ,ResultFileName );

			if( aPush < bPush ) return 2 ;
			else if( aPush > bPush ) return 1 ;
			else if( aHp.compareTo(bHp) < 0 ) return 2 ;
			else if( aHp.compareTo(bHp) > 0 ) return 1 ;
			else return -1 ;
		}
		
		// 片方が生存ならば戦闘終了
		else if( aFlag && !bFlag ) return 2 ;
		else if( bFlag && !aFlag ) return 1 ;

		
		// 全滅の場合の処理
		else{
			Util.outputString("  --判定：　<プッシュ>："+a.getTotalPush()+"　- "+b.getTotalPush()+", HP:"+a.getTotalOverKillDamage()+" - "+b.getTotalOverKillDamage()+"\n" ,ResultFileName );
			if( a.getTotalPush() < b.getTotalPush() ) return 2 ;
			else if( a.getTotalPush() > b.getTotalPush() ) return 1 ;
			else if( a.getTotalOverKillDamage().compareTo( b.getTotalOverKillDamage() ) < 0 ) return 2 ;
			else if( a.getTotalOverKillDamage().compareTo( b.getTotalOverKillDamage() ) > 0 ) return 1 ;
			else return -1 ;
		}
		
	}
	

	/**
	 * 全てのユニットに対して、毒ダメージを与えます。
	 * @param a
	 * @param b
	 */
	protected void checkPoison( Party a, Party b ){
		ArrayList<Player> aMember = a.getMember() ;
		ArrayList<Player> bMember = b.getMember() ;
		boolean flag = false ;
		for(int i=0;i<aMember.size();i++){
			if( flag ) break ;
			if( aMember.get(i).getPoison() > 0 || aMember.get(i).getDeadly() > 0  )
				flag = true ;
		}
		for(int i=0;i<bMember.size();i++){
			if( flag ) break ;
			if( bMember.get(i).getPoison() > 0 || bMember.get(i).getDeadly() > 0 )
				flag = true ;
		}
		
		// 誰かが毒なら
		if( flag ){
			Util.outputString("  --毒判定。\n" ,ResultFileName );
			for(int i=0;i<aMember.size();i++){
				if( aMember.get(i).getDeadly() > 0 )
					aMember.get(i).addPoison( aMember.get(i).getDeadly() ) ;
				if( aMember.get(i).getPoison() > 0 )
					aMember.get(i).damage( BigInteger.valueOf(aMember.get(i).getPoison()), turn ) ;
			}
			for(int i=0;i<bMember.size();i++){
				if( bMember.get(i).getDeadly() > 0 )
					bMember.get(i).addPoison( bMember.get(i).getDeadly() ) ;
				if( bMember.get(i).getPoison() > 0 )
					bMember.get(i).damage( BigInteger.valueOf(bMember.get(i).getPoison()), turn ) ;
			}
		}
	}

	/**
	 * 両partyのユニットに行動権を与えます。
	 * その際に、同時行動で使用するアクションを保存します。
	 * @param a
	 * @param b
	 */
	private boolean entitleAct( Party a, Party b ){
		ArrayList<Player> aMember = a.getMember() ;
		ArrayList<Player> bMember = b.getMember() ;
		ArrayList<Player> actedPlayers = new ArrayList<Player>();

		
		// 全員のAP・サブAPが0なら、ターン終了
		// APが0のユニットは行動しない
		boolean endFlag = true ;
		for(int i=0;i<aMember.size();i++){
			Player p = aMember.get(i) ;
			if( p.getAp().add(p.getSubAp()).compareTo(BigInteger.ZERO) > 0 )
				endFlag = false ;
			else 
				actedPlayers.add(p);

		}
		for(int i=0;i<bMember.size();i++){
			Player p = bMember.get(i) ;
			if( p.getAp().add(p.getSubAp()).compareTo(BigInteger.ZERO) > 0 )
				endFlag = false ;
			else 
				actedPlayers.add(p);
		}
		if( endFlag )
			return true ;
		
		while( actedPlayers.size() < aMember.size()+bMember.size() ){
			// 行動するユニットを選ぶ
			// クイック-スロー、素早さの最大値を取得
			BigInteger maxSpd = BigInteger.ZERO ;
			int maxQuick = Integer.MIN_VALUE ;

			for(int i=0;i<aMember.size();i++){
				Player p = aMember.get(i);
				if( !actedPlayers.contains(p) ){
					if( p.getQuick() > maxQuick ){
						maxQuick = p.getQuick() ;
						maxSpd = p.getSpd() ;
					}
					else if( p.getQuick() == maxQuick && maxSpd.compareTo(p.getSpd()) < 0 )
						maxSpd = p.getSpd() ;
				}
			}
			for(int i=0;i<bMember.size();i++){
				Player p = bMember.get(i);
				if( !actedPlayers.contains(p) ){
					if( p.getQuick() > maxQuick ){
						maxQuick = p.getQuick() ;
						maxSpd = p.getSpd() ;
					}
					else if( p.getQuick() == maxQuick && maxSpd.compareTo(p.getSpd()) < 0 )
						maxSpd = p.getSpd() ;
				}
			}

			// 使用する剣の選択
			boolean flag = false ;
			for(int i=0;i<aMember.size();i++)
				flag |= aMember.get(i).setUseAction(maxSpd, maxQuick, turn,a,b);
			for(int i=0;i<bMember.size();i++)
				flag |= bMember.get(i).setUseAction(maxSpd, maxQuick, turn,b,a);
			
			// 誰も剣を使う人がいなければAP0に
			if( !flag ){
				for(int i=0;i<aMember.size();i++){
					Player p = aMember.get(i) ;
					if( p.getSpd().compareTo(maxSpd) == 0 && p.getQuick() == maxQuick
					 && !actedPlayers.contains(p) ){
						p.setAp(BigInteger.ZERO);
						actedPlayers.add(p);
					}
				}
				for(int i=0;i<bMember.size();i++){
					Player p = bMember.get(i) ;
					if( p.getSpd().compareTo(maxSpd) == 0 && p.getQuick()== maxQuick
					 && !actedPlayers.contains(p) ){
						p.setAp(BigInteger.ZERO);
						actedPlayers.add(p);
					}
				}
				Util.outputString("    --使える剣がない。\n\n" ,SettingMatch.ResultFileName);
			// 誰かが剣を使ったら仕切り直し
			} else 
				return false ;
		}
		return true ;

	}
	
	/**
	 * このタイミングで同時行動が発生するユニットの配列を各パーティごとに返します。
	 * @param aParty
	 * @param bParty
	 * @return [a側の同時行動剣師群,b側の同時行動剣師群]
	 */
	@SuppressWarnings("unchecked")
	protected ArrayList<Player>[] getActionPlayersList( ArrayList<Player> aParty, ArrayList<Player> bParty ){
		ArrayList<Player>[] result = new ArrayList[2] ;
		result[0] = new ArrayList<Player>() ;
		result[1] = new ArrayList<Player>() ;
		BigInteger maxSpd = BigInteger.ZERO ;
		int maxQuick = Integer.MIN_VALUE ;
		
		BigInteger aAp[] = getPartyAps( aParty ) ;
		BigInteger bAp[] = getPartyAps( bParty ) ;
		
		// 行動するユニットを選ぶ
		// クイック-スロー、素早さの最大値を取得
		for(int i=0;i<aAp.length;i++){
			if( aAp[i].compareTo(BigInteger.ZERO) > 0 ){
				if( aParty.get(i).getQuick() > maxQuick ){
					maxQuick = aParty.get(i).getQuick() ;
					maxSpd = aParty.get(i).getSpd() ;
				}
				else if( aParty.get(i).getQuick() == maxQuick
					  && maxSpd.compareTo(aParty.get(i).getSpd()) < 0 )
					maxSpd = aParty.get(i).getSpd() ;
			}
			
		}
		for(int i=0;i<bAp.length;i++){
			if( bAp[i].compareTo(BigInteger.ZERO) > 0 ){
				if( bParty.get(i).getQuick() > maxQuick ){
					maxQuick = bParty.get(i).getQuick() ;
					maxSpd = bParty.get(i).getSpd() ;
				}
				else if( bParty.get(i).getQuick() == maxQuick
					  && maxSpd.compareTo(bParty.get(i).getSpd()) < 0 )
					maxSpd = bParty.get(i).getSpd() ;
			}
		}

		// クイック-スロー、素早さが最大の剣師を選択
		for(int i=0;i<aAp.length;i++){
			if( aAp[i].compareTo(BigInteger.ZERO) > 0 && aParty.get(i).getSpd() == maxSpd
			 && aParty.get(i).getQuick() == maxQuick ){
				result[0].add(aParty.get(i)) ;
				aParty.get(i).setAp( aParty.get(i).getAp().subtract(BigInteger.ONE) ) ;
				Util.outputString("  --"+aParty.get(i).getComment()+", 残りAP:"+aAp[i]+"\n" ,ResultFileName );
			}
		}
		for(int i=0;i<bAp.length;i++){
			if( bAp[i].compareTo(BigInteger.ZERO) > 0 && bParty.get(i).getSpd() == maxSpd
			 && bParty.get(i).getQuick() == maxQuick ){
				result[1].add(bParty.get(i)) ;
				bParty.get(i).setAp( bParty.get(i).getAp().subtract(BigInteger.ONE) ) ;
				Util.outputString("  --"+bParty.get(i).getComment()+", 残りAP:"+bAp[i]+"\n" ,ResultFileName );
			}
		}
		return (ArrayList<Player>[])result ;
	}



	/**
	 * Party内のAPの一覧を配列にして返します。
	 * @return
	 */
	private BigInteger[] getPartyAps( ArrayList<Player> party ){
		BigInteger[] result = new BigInteger[party.size()] ;
		for(int i=0;i<party.size();i++)
			result[i] = party.get(i).getAp() ;
		return result ;
	}

	// 戦闘結果をファイル出力します。
	protected void outputResult() throws Exception{
	}
}
